@@ -55,10 +55,25 @@ class StefUtil:
         self.dset_dir = dataset_dir
         self.model_dir = model_dir
 
+        self.proj_path = os_join(self.base_path, self.proj_dir)
+        self.dset_path = os_join(self.base_path, self.dset_dir)
+        self.model_path = os_join(self.base_path, self.model_dir)
         self.plot_path = os_join(self.base_path, self.proj_dir, 'plot')
+        self.eval_path = os_join(self.base_path, self.proj_dir, 'eval')
         os.makedirs(self.plot_path, exist_ok=True)
+        os.makedirs(self.eval_path, exist_ok=True)
 
-    def save_fig(self, title, save=True):
+    def save_fig(self, title, save=True, prefix_time: bool = True):
+        """
+        :param title: Rendered figure title
+        :param save: If true, figure is saved to project plot directory
+            No effect otherwise
+        :param prefix_time: If true, timestamp is prefixed before filename
+            Otherwise, timestamp is appended to the end
+        """
         if save:
-            fnm = f'{title}, {now(for_path=True)}.png'
+            if prefix_time:
+                fnm = f'{now(for_path=True)}_{title}.png'
+            else:
+                fnm = f'{title}, {now(for_path=True)}.png'
             plt.savefig(os_join(self.plot_path, fnm), dpi=300)
